<?php
/**
 * Plugin Name: LTL Freight Quotes - Freightview Edition
 * Plugin URI: https://eniture.com/products/
 * Description: Dynamically retrieves your negotiated shipping rates from Freightview and displays the results in the WooCommerce shopping cart.
 * Version: 1.1.2
 * Author: Eniture Technology
 * Author URI: http://eniture.com/
 * Text Domain: eniture-technology
 * License: GPL version 2 or later - http://www.eniture.com/
 * WC requires at least: 6.4
 * WC tested up to: 8.3.1
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

require_once 'vendor/autoload.php';

define('EN_FREIGHTVIEW_MAIN_DIR', __DIR__);
define('EN_FREIGHTVIEW_MAIN_FILE', __FILE__);

add_action('before_woocommerce_init', function () {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class)) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    }
});

if (empty(\EnFreightviewGuard\EnFreightviewGuard::en_check_prerequisites('Freightview', '5.6', '4.0', '2.3'))) {
    require_once 'en-install.php';
}

/**
 * Load scripts for Freightview Freight json tree view
 */
if (!function_exists('en_freightview_jtv_script')) {
    function en_freightview_jtv_script()
    {
        wp_register_style('en_freightview_json_tree_view_style', plugin_dir_url(__FILE__) . 'admin/tab/logs/en-json-tree-view/en-jtv-style.css');
        wp_register_script('en_freightview_json_tree_view_script', plugin_dir_url(__FILE__) . 'admin/tab/logs/en-json-tree-view/en-jtv-script.js', ['jquery'], '1.0.0');

        wp_enqueue_style('en_freightview_json_tree_view_style');
        wp_enqueue_script('en_freightview_json_tree_view_script', [
            'en_tree_view_url' => plugins_url(),
        ]);
    }

    add_action('admin_init', 'en_freightview_jtv_script');
}





